#!/usr/bin/python
# -*- coding: utf-8 -*-

"""Ce programme permet de lancer l'asservissement sur la pyboard
"""

__author__ = "Javier OJEDA"
__license__ = "GPL"
__version__ = "1.0.0"
__maintainer__ = "Javier OJEDA"
__email__ = "javier.ojeda@ens-cachan.fr"
__status__ = "Education"

###############################################################################
##
##    Correcteur RST sur Pyboard
##
###############################################################################


# Load des modules complmentaires
from pyb import Timer, Pin, ADC, DAC

tim = Timer(1, freq=1000) # Dfinition du timer 1  la frquence 1000Hz

adc_pos = ADC(Pin('X11')) # Convertisseur analogique-numrique sur la pin 11

adc_neg = ADC(Pin('X12')) # Convertisseur analogique-numrique sur la pin 12

dac = DAC(Pin('X5'), bits=12)   # Convertisseur CNA 12 bit sur la pin X5

erreur_corrigee_old = 0 # Initialisation des variables

erreur_old = 0 # Initialisation des variables

# Paramtres du correcteur RST

S_1 = 0.64484225
R_0 = 4.5400731
R_1 = -0.97949111

# Initialisation
tick = 0
echeance = 0

# Fonction ralisation le correcteur RST avec saturation
def correction():
    
    # Dfinition des variables globales accessible en dehors de la fonction
    global erreur_corrigee_old
    global erreur_old
    
    # Conversion numrique de la tension de sortie du systme
    entree_pos = adc_pos.read() # Lecture du CAN
    entree_neg = adc_neg.read() # Lecture du CAN
    
    # Mesure de l'erreur
    erreur = entree_pos - entree_neg
    
    # Algorithme du correcteur RST avec T=1
    erreur_corrigee = -S_1*erreur_corrigee_old + R_0*erreur + R_1*erreur_old
    
    erreur_corrigee_int = int(erreur_corrigee) # float -> int pour le DAC
    
    # Saturation de la sortie
    
    if (erreur_corrigee_int < 0):
        erreur_corrigee_int = 0
        
    if (erreur_corrigee_int > 4095):
        erreur_corrigee_int = 4095
    
    dac.write(erreur_corrigee_int) # Ecriture sur le CNA

    # Enregistrement des donnes pour le calcul rcursif
    erreur_corrigee_old = erreur_corrigee_int

    erreur_old = erreur
    
# Fonction pour la gestion de l'echantillonnage
def gestion_echantillon(timer):
    global tick
    tick += 1
    
    
# Fonction qui se dclenche  chaque fois que le timer fini sa boucle
tim.callback(gestion_echantillon)

# Programme principal
while True:
    
    if tick == echeance:
        echeance += 1
        correction()
    elif tick == 2**24: # Remise  0 des compteurs tous les 2**24 tops
        echeance = 0
        tick == 0

